@@ -1,4 +1,3 @@
-
 from collections import Counter
 import math
 
@@ -41,7 +40,6 @@ def get_inverse_document_frequency(siglist):
     N = len(siglist)
     document_frequency = get_document_frequency(siglist)
     inverse_document_frequency = {
-        k: math.log(
-            N / v) for k,
-        v in document_frequency.items()}
+        k: math.log(N / v) for k, v in document_frequency.items()
+    }
     return inverse_document_frequency
